{
*****************************************************************************
*  Name    : RandGen.BAS                                                    *
*  Author  : Ahmed Lazreg (Octal)                                           *
*            octal@pocketmt.com   http://www.pocketmt.com                   *
*  Notice  : Copyright (c) 2007                                             *
*          : All Rights Reserved                                            *
*  Date    : 06/10/2007                                                     *
*  Version : 1.0                                                            *
*                                                                           *
*  Notes   : A Rudimentary Pseudo Random Number Generator (Modulo based)    *
*                                                                           *
*  Usage   : Call Initialize() to initialize the Initial value of the       *
*            generator. The generator gives better values when the initial  *
*            seed value is a Prime Number.                                  *
*            For the same Initial Seed, you will get the same serie of      *
*            generated values. This let you repeat some experiences (and    *
*            this is why it's called a PSEUDO-random number generator.      *
*            If you need an automatic different initial value each time you *
*            start the number generator, you can set the initial value to   *
*            the read of an ADC value on a FLOATING Analog PIN of a PIC.    *
*            Call Rand() to get/generate a new random value                 *
*                                                                           *
*            You can try to change the Magic Values to change the           *
*            Pseudo-Random Number Generator Behaviour                       *
*                                                                           *
*****************************************************************************
}

Module RandGen

Const MagicA = 7,
      MagicB = 7,
      MagicC = 255

Private Dim Seed As Byte

{
****************************************************************************
* Name    : Rand()                                                         *
* Purpose : Return a new Pseudo Random Number each time called             *
****************************************************************************
}
Public Function Rand() As Byte
    Seed = (MagicA * Seed + MagicB) Mod MagicC
    result = Seed
End Function

{
****************************************************************************
* Name    : Rand()                                                         *
* Purpose : Initialize the Random number generator                         *
*           The initial value could be a Value read from a Floating Analog *
*           PIC Pin.
****************************************************************************
}
Public Sub Initialize(ByVal InitialSeed As Byte)
    Seed = InitialSeed
    Seed = Rand()
End Sub
